@@ -4,6 +4,12 @@ from .SchemaType import SchemaType
 
 
 class SchemaPrimitive(SchemaType):
+    """
+    A schema requiring a primitive variable type
+
+    :param python_type: The primitive Python type expected by this schema
+    """
+
     JSON_TYPES: Dict[Type, str] = {
         str: 'string',
         bool: 'boolean',
@@ -24,7 +30,7 @@ class SchemaPrimitive(SchemaType):
             value: Any,
             dom_info: Tuple = None
     ) -> Any:
-        return self.python_type(value)
+        return super().__call__(self.python_type(value))
 
     @property
     def type_name(self) -> str:
