@@ -4,6 +4,7 @@ from typing import Any, Type, Iterator, Union, Mapping
 
 import inspect
 
+from ..mixins import RegistersSubclasses
 from ..base_schema import Schema
 from ..object_schema import SchemaObject, SchemaAnyOf
 from ..dom import (
@@ -16,21 +17,36 @@ from .UserProperty import UserProperty
 
 
 class UserProperties(DOMProperties):
+    """
+    A container for property information for a :class:`.UserObject` subclass.
+    """
 
     def __init__(
             self,
             user_class: Type[UserObject],
             additional_properties: Union[bool, Schema] = False
     ):
+        """
+        :param user_class:            The subclass of :class:`.UserObject` to extract properties from.
+                                      Properties will be extracted from the class's
+                                      :class:`~wysdom.user_objects.UserProperty` descriptors.
+        :param additional_properties: Defines whether a :class:`.DOMObject` permits additional
+                                      dynamically-named properties. Can be True or False, or
+                                      can be set to a specific :class:`~wysdom.base_schema.Schema`
+                                      to restrict the permitted types of any additional properties.
+        """
         self._user_class = user_class
         properties = {}
+        required = set()
         for superclass in reversed(list(self._schema_superclasses())):
             for k, v in superclass.__dict__.items():
                 if isinstance(v, UserProperty):
                     if not v.name:
                         v.name = k
                     properties[v.name] = v.schema_type
-        super().__init__(properties, additional_properties)
+                    if not v.optional:
+                        required.add(v.name)
+        super().__init__(properties, required, additional_properties)
 
     def _schema_superclasses(self) -> Iterator[Type[UserObject]]:
         for superclass in inspect.getmro(self._user_class):
@@ -89,15 +105,25 @@ class UserObject(DOMObject):
 
     @classmethod
     def __json_schema__(cls) -> Schema:
-        if cls.registered_subclasses():
+        has_subclasses = False
+        if issubclass(cls, RegistersSubclasses):
+            if cls.registered_subclasses():
+                has_subclasses = True
+        if has_subclasses:
             return SchemaAnyOf(
-                subclass.__json_schema__()
-                for subclass in cls.registered_subclasses().values()
-                if issubclass(subclass, UserObject)
+                (
+                    subclass.__json_schema__()
+                    for subclass in cls.registered_subclasses().values()
+                    if issubclass(subclass, UserObject)
+                    and not isinstance(subclass.__json_schema__(), SchemaAnyOf)
+                ),
+                schema_ref_name=f"{cls.__module__}.{cls.__name__}"
             )
         else:
             return SchemaObject(
                 properties=cls.__json_schema_properties__.properties,
+                required=cls.__json_schema_properties__.required,
                 additional_properties=cls.__json_schema_properties__.additional_properties,
-                object_type=cls
+                object_type=cls,
+                schema_ref_name=f"{cls.__module__}.{cls.__name__}"
             )
